import java.awt.*;
import java.awt.event.*;

/**
   This class is a self contained extended dialog box that is used to configure the
   keys used to imitate the joypad on the gameboy

   @author David Winchurch.
   @version 06/03/2001
**/

public class KeyConfigurator extends Dialog implements ActionListener, KeyListener
{
   //Arrays used to store the key codes. OLDKEYS stores the original set for undo purposes
   private int[] KEYS, OLDKEYS;

   //The buttons, panels and textfields used to display all the required key info
   private Button ok=new Button("OK"), cancel=new Button("Cancel");

   private Panel buttonPanel=new Panel(new GridLayout(1,2,20,20)),
                 configPanel=new Panel(new GridLayout(9,2,3,3)),
                 picturePanel;

   private TextField up=new TextField(), down=new TextField(),
                     left=new TextField(), right=new TextField(),
                     a=new TextField(), b=new TextField(),
                     start=new TextField(), select=new TextField(),
                     allButtons=new TextField();

   //Image used to store the picture of the gameboy displayed on the right
   private Image gameboy;

   /**
      Constructor. Creates the Dialog Box and adds all the required components to it.
      Creates and instantiates all the global values and objects

      @param f the parent frame
      @param keys pointer to the array the key codes will be stored in
   **/

   public KeyConfigurator(Frame f, int[] keys, Image im)
   {
      //calls the DialogBox constuctor to set the parent, title and modality
      super(f, "Control Config", true);

      //KEYS becomes a pointer to keys in the class which created this instance and so
      //modifications to KEYS automatically changes keys
      KEYS=keys;
      gameboy=im;

      //OLDKEYS has to be created differently so that it copies the values out of keys
      //and does not get modified by changes to either KEYS or keys
      OLDKEYS=new int[keys.length];
      for(int i=0; i<OLDKEYS.length; i++) OLDKEYS[i]=keys[i];

      //Dialog Box setup
      setLayout(new BorderLayout());
      setLocation(f.getLocation().x+f.getSize().width ,f.getLocation().y);
      setBackground(Color.lightGray);

      //None of the textFields should be editable
      up.setEditable(false); down.setEditable(false);
      left.setEditable(false); right.setEditable(false);
      a.setEditable(false); b.setEditable(false);
      start.setEditable(false); select.setEditable(false);
      allButtons.setEditable(false);

      //Add action/Item Listeners to components
      ok.addActionListener(this); cancel.addActionListener(this);
      up.addKeyListener(this); down.addKeyListener(this);
      left.addKeyListener(this); right.addKeyListener(this);
      a.addKeyListener(this); b.addKeyListener(this);
      start.addKeyListener(this); select.addKeyListener(this);
      allButtons.addKeyListener(this);

      //Set the text displayed in the textfields
      up.setText(KeyEvent.getKeyText(KEYS[0]));
      down.setText(KeyEvent.getKeyText(KEYS[1]));
      left.setText(KeyEvent.getKeyText(KEYS[2]));
      right.setText(KeyEvent.getKeyText(KEYS[3]));
      a.setText(KeyEvent.getKeyText(KEYS[4]));
      b.setText(KeyEvent.getKeyText(KEYS[5]));
      start.setText(KeyEvent.getKeyText(KEYS[6]));
      select.setText(KeyEvent.getKeyText(KEYS[7]));
      allButtons.setText(KeyEvent.getKeyText(KEYS[8]));

      //Setting up the picturePanel and extending some methods
      picturePanel=new Panel()
      {
         /**
            Overwritten paint method. Ensures that the correct image is drawn
            when the program is obscured then revealed

            @param g the specified context to use for updating.
         **/
         public void paint(Graphics g) {g.drawImage(gameboy,0,0,this);}

         /**
            Required for pack methods to work

            @returns the prefered size of the component as a Dimension
         **/

         public Dimension getPreferredSize() {return new Dimension(gameboy.getWidth(this),gameboy.getHeight(this));}
      };
      picturePanel.setBackground(new Color(174, 168, 217));

      //Add the relevant components to the configPanel
      configPanel.setBackground(new Color(174, 168, 217));
      configPanel.add(new Label("Up")); configPanel.add(up);
      configPanel.add(new Label("Down")); configPanel.add(down);
      configPanel.add(new Label("Left")); configPanel.add(left);
      configPanel.add(new Label("Right")); configPanel.add(right);
      configPanel.add(new Label("A")); configPanel.add(a);
      configPanel.add(new Label("B")); configPanel.add(b);
      configPanel.add(new Label("Start")); configPanel.add(start);
      configPanel.add(new Label("Select")); configPanel.add(select);
      configPanel.add(new Label("All Buttons")); configPanel.add(allButtons);

      //Add the relevant components to the buttonPanel
      buttonPanel.setBackground(new Color(174, 168, 217));
      buttonPanel.add(ok);
      buttonPanel.add(cancel);

      //Add all the panels to the dialog box
      add(configPanel, BorderLayout.WEST);
      add(picturePanel, BorderLayout.CENTER);
      add(buttonPanel, BorderLayout.SOUTH);

      //resizes the window to its optimum size
      pack();

      //Displays it
      setVisible(true);

   }

   /**
      This is the ActionEvent handler for KeyConfigurator. It accepts events generated
      by any component with an ActionListener added.

      @param event The event that was generated.
   **/

   public void actionPerformed(ActionEvent event)
   {
      //gets the object that generated the ActionEvent for identification purposes
      Object source=event.getSource();

      if(source==ok) this.dispose();
      //undoes and changes made to the keys array
      if(source==cancel) {for(int i=0; i<KEYS.length; i++) KEYS[i]=OLDKEYS[i]; this.dispose();}
   }

   /**
      This is the key pressed event handler for GameBoyEmu. It accepts KeyEvents generated
      by pressing a key when any component with a KeyListener has focus

      @param k The KeyEvent that was generated.
   **/

   public void keyPressed(KeyEvent e)
   {
      //gets the object that generated the KeyEvent for identification purposes
      Object source=e.getSource();
      //gets the integer corresponding to the pressed key
      int keyPress=e.getKeyCode();

      //The following if statements find out which textfield the key was pressed in, sets the textfields text
      //and sets the key code corresponding to the gameboy joypad
      if(source==up) {KEYS[0]=keyPress; up.setText(KeyEvent.getKeyText(keyPress)); up.transferFocus();}
      if(source==down) {KEYS[1]=keyPress; down.setText(KeyEvent.getKeyText(keyPress)); down.transferFocus();}
      if(source==left) {KEYS[2]=keyPress; left.setText(KeyEvent.getKeyText(keyPress)); left.transferFocus();}
      if(source==right) {KEYS[3]=keyPress; right.setText(KeyEvent.getKeyText(keyPress)); right.transferFocus();}

      if(source==a) {KEYS[4]=keyPress; a.setText(KeyEvent.getKeyText(keyPress)); a.transferFocus();}
      if(source==b) {KEYS[5]=keyPress; b.setText(KeyEvent.getKeyText(keyPress)); b.transferFocus();}
      if(source==start) {KEYS[6]=keyPress; start.setText(KeyEvent.getKeyText(keyPress)); start.transferFocus();}
      if(source==select) {KEYS[7]=keyPress; select.setText(KeyEvent.getKeyText(keyPress)); select.transferFocus();}

      if(source==allButtons) {KEYS[8]=keyPress; allButtons.setText(KeyEvent.getKeyText(keyPress)); allButtons.transferFocus();}
   }

   /*
      The following methods are here because they are required when a KeyListener is
      implemented. They do nothing in this program

      @param e The KeyEvent that was generated.
   */

   public void keyReleased(KeyEvent e) {}
   public void keyTyped(KeyEvent e) {}
} 